<?php declare(strict_types=1);

namespace Dump {

    use Psr\Http\Server\MiddlewareInterface,
        Psr\Http\Message\ServerRequestInterface,
        Psr\Http\Server\RequestHandlerInterface,
        Psr\Http\Message\ResponseInterface;

    use Laminas\Diactoros\{ Response, Stream, Response\JsonResponse, Response\HtmlResponse, Response\TextResponse };

    use function stream_copy_to_stream;

    class DumpMiddleware implements MiddlewareInterface
    {
        /**
         * @var [class]
         */
        protected static $dump_stack = [];

        /**
         * Processing the dump stack accumulated in the request
         * @param  ServerRequestInterface  $request
         * @param  RequestHandlerInterface $handler
         * @return ResponseInterface
         */
        public function process(ServerRequestInterface $request, RequestHandlerInterface $handler) : ResponseInterface
        {
            $response = $handler->handle($request);

            if ( empty(static::$dump_stack) ) {
                return $response;
            }

            $stream = new class("php://memory", "rw") extends Stream {
                public function append_resource($resource) {
                    stream_copy_to_stream($resource, $this->resource);
                }
            };

            switch (true) {
                case $response instanceof TextResponse:
                    $body = $response->getBody();

                    foreach(static::$dump_stack as $item) {
                        $stream->write($item->renderText());
                    }

                    $stream->append_resource($body->detach());
                    break;

                case $response instanceof JsonResponse:
                    foreach(static::$dump_stack as $item) {
                        $dump[] = $item->renderArray();
                    }

                    $stream->write(json_encode(array_merge([ "_dump" => $dump ?? [] ], json_decode($response->getBody()->getContents() ?? "{}", true)), JsonResponse::DEFAULT_JSON_FLAGS));
                    break;

                default:
                case $response instanceof Response:
                    $body = $response->getBody();

                    foreach(static::$dump_stack as $item) {
                        $stream->write($item->renderHtml());
                    }

                    $stream->append_resource($body->detach());
                    break;
            }

            return $response->withBody($stream);
        }

        public static function dump(...$what)
        {
            static::$dump_stack[] = $dump = new Dump(...$what);
            return $dump;
        }
    }
}

namespace {
    if (! function_exists('dump') ) {
        function dump(...$what) {
            return \Dump\DumpMiddleware::dump(...$what);
        }
    }
}
